# -*- coding: utf-8 -*-
"""
author: F.K. Unseld

This is the cleaned up version of the script used to generate Fig 3 in the 
paper "On-chip Integration of Si/SiGe-based Quantum Dots and Switched-capacitor 
Circuits". 

The references (solid lines) can be found in '../Fig3/References/' the name of
the CSV file indicates the voltage on the T-Gate. 

In the folder '../Fig3/PulsedTGate/' is the time data which was recorded 
while the ac signal was applied on the T-Gate. It consists of the following 
phases: 
        - a short start up (~12sec)
        - repeated sequence
            - turning the transistor off
            - applying the ac pulse
            - turning the transistor on

The switching of the transistor comprises around 60 large peaks. This script 
will first detect those peaks, and isolate the times of interest where the
ac signal was applied. Using a cross correlation between a raference signal 
and the isolatet signal we try to separate the positive from the negative 
pulse part. To match which pulse part is which we compare the mean values 
to the previously recorded values and to the references and try to make a 
decision based on that. This procedure isn't 100% accurate and we need to
correct a few mistakes in the end. 
"""



import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import (inset_axes, mark_inset)
from matplotlib.ticker import EngFormatter,StrMethodFormatter
from matplotlib.ticker import MaxNLocator
from matplotlib.patches import ConnectionPatch
from scipy.signal import argrelextrema, find_peaks
import os


#%%
# Import Pulsed T Gate File (since it's a time series 
# data_Time (600s long))) and process it such that it is in a conveniant 
# shape for further processing. 


data_Time = pd.read_csv('../Fig3/PulsedTGate/ACTGate.csv')
data_Time = data_Time.transpose()
data_Time.rename({data_Time.columns[0]: 'data'}, axis=1, inplace = True)
data_Time_orig = data_Time
data_Time.index = data_Time_orig.index.astype("float")
#%%
# Import the references from the CSV files
Ref_datas = []
for Reffile in os.listdir('../Fig3/References'):
    tmp = pd.read_csv('../Fig3/References/'+Reffile)
    tmp =  tmp.set_index(tmp.columns[0])
    Ref_datas.append(tmp)
#%% Peak Finding
    
peaks = find_peaks(abs(data_Time.data.values-data_Time.data.rolling(250).mean()),
           prominence = 0.2e-9, 
           distance = 35,width = [1,3])
ilocs_max = peaks[0]
data_Time['maxi'] = data_Time.data.iloc[ilocs_max]

fig,ax = plt.subplots(nrows = 1, ncols = 1, dpi = 450, figsize=(5.4,4))
data_Time.data.plot(ax=ax, label='time data')
ax.set_xlabel('Time (s)')
ax.set_ylabel('Current (A)')

# Manually defining the starting Point of the switching process of the 
# transistor and using the time after the 59th maximum to isolate the first
# switching region. 
data_Time_tC = data_Time[12.32:]

# SwitchingDataFrames is a list of dataframes in which the ac signal applied
# The exact location of which however is not yet determined. 
SwitchingDataFrames = []

Start = data_Time_tC.dropna(0,subset = ['maxi']).iloc[59].name+0.01
Stop =  data_Time_tC.dropna(0,subset = ['maxi']).iloc[59].name+0.2

SwitchingDataFrames.append(data_Time_tC[Start:Stop])

data_Time_tC.data[Start:Stop].plot(ax=ax, c='C1', label = 'AC pulses',linewidth = 0.4)
data_Time_tC = data_Time_tC[Stop:]
ax.legend()
while Stop<580: # No relevant data after t_stop = 580s
    # Use Time after 119th Maximum for switching
    Start = data_Time_tC.dropna(0,subset = ['maxi']).iloc[119].name+0.03
    Stop =  data_Time_tC.dropna(0,subset = ['maxi']).iloc[119].name+0.20
    
    # Check if last peak is downwards wrt. a reference point 5 samples later (
    # we know last peak before AC signal is going down)
    A = data_Time_tC.data[round(data_Time_tC.dropna(0,subset = ['maxi']).iloc[119].name+0.005,3)]
    B = data_Time_tC.dropna(0,subset = ['maxi']).iloc[119]['data']
    # If the peak is going upwards, try the peak before, keep trying
    i = 118
    while A<B:
        Start = data_Time_tC.dropna(0,subset = ['maxi']).iloc[i].name+0.03
        Stop =  data_Time_tC.dropna(0,subset = ['maxi']).iloc[i].name+0.20
        A = data_Time_tC.data[round(data_Time_tC.dropna(0,subset = ['maxi']).iloc[i].name+0.005,3)]
        B = data_Time_tC.dropna(0,subset = ['maxi']).iloc[i]['data']
        i = i-1
        # break condition
        if i<=100:
            print("SearchError")
            break
    # Append and clip
    SwitchingDataFrames.append(data_Time_tC[Start:Stop])
    ##
    data_Time_tC.data[Start:Stop].plot(ax=ax, c='C1', linewidth = 0.4)
    ##
    data_Time_tC  = data_Time_tC[Stop:]
#%% XCorrelation with Reference signal
# Define Reference signal for xcorr signal (XCS)
# It's frequency specific, in this case the ac pulse was 100Hz
XCS100 = np.zeros(100)
for i,j in enumerate(XCS100):
    if np.mod(i,5)==1:
        XCS100[i]=1

t780 = []
t790 = []
t800 = []

# Defining the respective V_in for every time we applied the ac signal
V_ins = np.round(np.arange(1.500,1.550,0.5e-3)[:len(SwitchingDataFrames)],4)

# Legacy Reasons
Ref_L = Ref_datas[0]
Ref_H = Ref_datas[2]

#Reindexing the reference signals to match V_in sampling rates
newindex = np.round(np.arange(min(Ref_H.index.values), max(Ref_H.index.values), 0.1e-3),4)
Ref_H = Ref_H.reindex(index = newindex)
Ref_H.interpolate(method = 'linear', inplace = True)

newindex = np.round(np.arange(min(Ref_L.index.values), max(Ref_L.index.values), 0.1e-3),4)
Ref_L = Ref_L.reindex(index = newindex)
Ref_L.interpolate(method = 'linear', inplace = True)

for idx,(V_in,df) in enumerate(zip(V_ins,SwitchingDataFrames)):
    # We can use the last 10 data points of each DF in SwitchingDataFrames
    # which is always after the ac signal, but before changing the transistor 
    # gate voltage. This gives the current when the T gate is at 790 mV, the 
    # DC bias. 
    t790.append([np.mean(df.data.iloc[-10:].values),np.std(df.data.iloc[-10:].values)])
    xcorr = np.correlate(abs(df.data.diff().values),XCS100,'valid')
    X = df.index.values[0:100]+np.nanargmax(xcorr)/1000
    tmp_ev=[]
    tmp_od=[]
    # Loop over the 19 or 20 valleys and peaks and save in even/odd signal,
    # before matching it to t780 and t800. 
    for i in range(19):
        if (i % 2) == 0:
            tmp_ev.append(df.data[np.round(X[i*5+2:i*5+4],3)].values)
        else:
            tmp_od.append(df.data[np.round(X[i*5+2:i*5+4],3)].values)
    # excluding some of the data points as they're part of the switching 
    # process and don't include useful information. 
    even = [np.mean(np.array(tmp_ev).reshape(np.size(tmp_ev))[5:-9]),np.std(np.array(tmp_ev).reshape(np.size(tmp_ev))[5:-9])]
    odd  = [np.mean(np.array(tmp_od).reshape(np.size(tmp_od))[5:-9]),np.std(np.array(tmp_od).reshape(np.size(tmp_od))[5:-9])]

    D1 = abs(even[0]-Ref_H.I_D[np.round(V_in,4)])+abs(odd[0]-Ref_L.I_D[np.round(V_in,4)])
    D2 = abs(even[0]-Ref_L.I_D[np.round(V_in,4)])+abs(odd[0]-Ref_H.I_D[np.round(V_in,4)])
    if idx<=1:
        if D1>D2:
            tmp780 = odd
            tmp800 = even            
        elif D2>D1:
            tmp780 = even
            tmp800 = odd   
        else:
            print("Sort error at {}".format(idx))
            tmp780 = odd
            tmp800 = even 
    elif idx>1:
        old800 = t800[-1][0]
        old780 = t780[-1][0]
        
        DD1 = abs(even[0]-old800)+abs(odd[0]-old780)
        DD2 = abs(even[0]-old780)+abs(odd[0]-old800)
        if D1>D2 and DD1<DD2: #closer to reference and smoother difference 
            tmp780 = odd
            tmp800 = even   
        elif D1<D2 and DD1>DD2:  #closer to reference and smoother difference 
            tmp780 = even
            tmp800 = odd 
        elif  D1>=D2 and DD1>=DD2: #closer to ref but larger diff
            tmp780 = odd
            tmp800 = even          
        elif  D1<=D2 and DD1<=DD2: #closer to ref but larger diff
            tmp780 = even
            tmp800 = odd 
        else:
            print("Sort error at {}".format(idx))
            tmp780 = odd
            tmp800 = even                
    t800.append(np.array(tmp800))
    t780.append(np.array(tmp780))

# In hindsight it seems the arrays are flipped
a = t800.copy()
t800 = np.array(t780)
t780 = np.array(a)


#%% Manual correction of wrongly matched Points
a = t800[7,:].copy()
(t800[7,:], t780[7,:])=(t780[7,:], a)

a = t800[19,:].copy()
(t800[19,:],t780[19,:])=(t780[19,:],a)

a = t800[28,:].copy()
(t800[28,:],t780[28,:])=(t780[28,:],a)

a = t800[39,:].copy()
(t800[39,:],t780[39,:])=(t780[39,:],a)

a = t800[47,:].copy()
(t800[47,:],t780[47,:])=(t780[47,:],a)

a = t800[59,:].copy()
(t800[59,:],t780[59,:])=(t780[59,:],a)

a = t800[66,:].copy()
(t800[66,:],t780[66,:])=(t780[66,:],a)

a = t800[78,:].copy()
(t800[78,:],t780[78,:])=(t780[78,:],a)

#%%Figure (Data Only)

fig,ax = plt.subplots(nrows = 1, ncols = 1, dpi = 450, figsize=(5.4,4))

Ref_labels = ["T = 780 mV","T = 790 mV","T = 800 mV"]
linewidth = 0.7
elinewidth = 0.5
glinewidth = 0.30
marker = 'x'
markersize = 2

colrs = ['C1','C0','C2']


for (colr,(idx,Ref_data)) in zip(colrs,enumerate(Ref_datas)):
    Ref_data[Ref_data.columns[0]].plot(
        ax=ax,
        style = colr+'--',
        linewidth = 0.7, 
        label = Ref_labels[idx])
    
x = np.arange(1.500,1.550,0.5e-3)[:len(t780)]
ax.errorbar(x,
               np.array(t780)[:,0],
               yerr=np.array(t780)[:,1], 
               C = 'C1', 
               ecolor = 'C1',
               ls = '-', 
               linewidth = linewidth,
               elinewidth = elinewidth,
               marker = marker,
               markersize = markersize)
ax.errorbar(x,
               np.array(t790)[:,0],
               yerr=np.array(t790)[:,1], 
               C = 'C0', 
               ecolor = 'C0',
               ls = '-', 
               linewidth = linewidth,
               elinewidth = elinewidth,
               marker = marker,
               markersize = markersize)

ax.errorbar(x,
               np.array(t800)[:,0],
               yerr=np.array(t800)[:,1], 
               C = 'C2',
               ecolor = 'C2',
               ls = '-', 
               linewidth = linewidth,
               elinewidth = elinewidth,
               marker = marker,
               markersize = markersize)


ax.minorticks_on()
ax.grid(which='major', linestyle='--', lw = glinewidth)
ax.grid(which='minor', linestyle=':', lw = glinewidth)

ax.legend([r'Reference $T=780\mathregular{mV}$',
        r'Reference $T=790\mathregular{mV}$',
        r'Reference $T=800\mathregular{mV}$',
        r'$T=790\mathregular{mV\ (DC)}-10\mathregular{mV\ (AC)}$',
        r'$T=790\mathregular{mV\ (DC)}\ +\ 0\mathregular{mV\ (AC)}$',
        r'$T=790\mathregular{mV\ (DC)}+10\mathregular{mV\ (AC)}$',
           'Static'])


ax.yaxis.set_major_formatter(EngFormatter(unit='A'))
ax.set_ylabel(r'$\mathregular{I_d}$')
ax.xaxis.set_major_locator(MaxNLocator(nbins=7))
ax.xaxis.set_major_formatter(EngFormatter(unit='V'))
ax.set_xlabel(r'$\mathregular{V_{in}}$')


ax.set_xlim((1.5,1.54))
ax.set_ylim((-50e-12,1e-9))

ax.set_xticks(np.arange(1.5,1.54,0.01))

fig.tight_layout(h_pad=1, w_pad=1)

#%% Figure (Data and Explanation Graph)


gridsize = (3, 1)
fig = plt.figure(dpi = 450, figsize=(3.54,4))
ax1 = plt.subplot2grid(gridsize, (0, 0), colspan=1, rowspan=1)
ax2 = plt.subplot2grid(gridsize, (1, 0), colspan=1, rowspan=2)

linewidth = 0.7
style = 'butt'

ax1.set_axis_off()
Vin_y = np.array([0,0,.5,.5,1,1])
Vin_x = np.array([0,1,1,2,2,3])

Vg_y = np.array([0,0,1,1,0,0])
Vg_x = np.array([0,1,1,3,3,20])/20

T_y = np.array([1,1,2,2,0,0,2,2,0,0,2,2, 0, 0, 2, 2, 0, 0, 2, 2, 0, 0, 1,1])/2
T_x = np.array([0,5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15,20])/20

Im_xb = np.array([15,15,18,18])/20
Im_yb = np.array([np.nan,1,1,np.nan])
Im_xo = np.array([6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15])/20
Im_yo = np.array([np.nan,1,1,np.nan,np.nan,1,1,np.nan,np.nan,1,1,np.nan,np.nan,1,1,np.nan,np.nan,1,1,np.nan,])
Im_xg = np.array([5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13,14,14])/20
Im_yg = np.array([np.nan,1,1,np.nan,np.nan,1,1,np.nan,np.nan,1,1,np.nan,np.nan,1,1,np.nan,np.nan,1,1,np.nan,])
Im_xk = np.array([0,3,3,5,5,18,18,19,19,20])/20
Im_yk = np.array([0,0,1,1,np.nan,np.nan,1,1,0,0])


grln_x = np.array([5,5,5,5,6,6,6,6,7,7,7,7,8,8,8,8,9,9,9,9,10,10,10,10,11,11,11,11,12,12,12,12,13,13,13,13,14,14,14,14,15,15,15,15,18,18,18,18])/20
grln_y = np.array([np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,])

ax1.plot(Vin_x,Vin_y+2,'k',lw= linewidth)
ax1.plot(Vg_x,Vg_y,'k',lw= linewidth)
ax1.plot(Vg_x+1,Vg_y,'k',lw= linewidth)
ax1.plot(Vg_x+2,Vg_y,'k',lw= linewidth)

ax1.plot(T_x,T_y-2,'k',lw= linewidth)
ax1.plot(T_x+1,T_y-2,'k',lw= linewidth)
ax1.plot(T_x+2,T_y-2,'k',lw= linewidth)

ax1.plot(Im_xk, Im_yk-4,'k',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xb, Im_yb-4,lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xo, Im_yo-4,lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xg, Im_yg-4,lw= linewidth, solid_capstyle=style)

ax1.plot(Im_xk+1, Im_yk-4,'k',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xb+1, Im_yb-4,'C0',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xo+1, Im_yo-4,'C1',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xg+1, Im_yg-4,'C2',lw= linewidth, solid_capstyle=style)

ax1.plot(Im_xk+2, Im_yk-4,'k',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xb+2, Im_yb-4,'C0',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xo+2, Im_yo-4,'C1',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xg+2, Im_yg-4,'C2',lw= linewidth, solid_capstyle=style)

ax1.plot(grln_x,grln_y,':', c = 'silver',lw=linewidth/2)

ax1.annotate(r'$\mathregular{V_{in}}$',(-.3,2.25), annotation_clip=False,size=10)
ax1.annotate(r'$\mathregular{V_{g}}$',(-.3,.25), annotation_clip=False,size=10)
ax1.annotate(r'$\mathregular{T}$',(-.3,-1.75), annotation_clip=False,size=10)
ax1.annotate(r'$\mathregular{I_{m}}$',(-.3,-3.75), annotation_clip=False,size=10)

ax1.annotate('1500 mV',(0.2,2.2), annotation_clip=False,size=5)
ax1.annotate('1501 mV',(1.2,2.7), annotation_clip=False,size=5)
ax1.annotate('1502 mV',(2.2,3.2), annotation_clip=False,size=5)

ax1.annotate('2200 mV', (4/20+2,.75), annotation_clip=False,size=5 )
ax1.annotate('1600 mV', (3+1/20,-.2), annotation_clip=False,size=5 )

ax1.annotate('800 mV',(3+1/20,-1.2), annotation_clip=False,size=5)
ax1.annotate('790 mV',(3+1/20,-1.75), annotation_clip=False,size=5)
ax1.annotate('780 mV',(3+1/20,-2.3), annotation_clip=False,size=5)

ax1.set_xlim(0,3.5)

for i,Ref_data in enumerate(Ref_datas):
    if (Ref_data.mean()<1e-6).all():
        Ref_datas[i] = Ref_data*1e9



linewidth = .5
elinewidth = .4
marker = 'x'
markersize = 0
colrs = ['C1','C0','C2']



for (colr,(idx,Ref_data)) in zip(colrs,enumerate(Ref_datas)):
    Ref_data[Ref_data.columns[0]].plot(
        ax=ax2,
        style = colr+'--',
        linewidth = 0.7, 
        label = Ref_labels[idx])
    
x = np.arange(1.500,1.550,0.5e-3)[:len(t780)]
ax2.errorbar(x,
               np.array(t780)[:,0]*1e9,
               yerr=np.array(t780)[:,1]*1e9, 
               C = 'C1', 
               ecolor = 'C1',
               ls = '-', 
               linewidth = linewidth,
               elinewidth = elinewidth,
               marker = marker,
               markersize = markersize)
ax2.errorbar(x,
               np.array(t790)[:,0]*1e9,
               yerr=np.array(t790)[:,1]*1e9, 
               C = 'C0', 
               ecolor = 'C0',
               ls = '-', 
               linewidth = linewidth,
               elinewidth = elinewidth,
               marker = marker,
               markersize = markersize)

ax2.errorbar(x,
               np.array(t800)[:,0]*1e9,
               yerr=np.array(t800)[:,1]*1e9, 
               C = 'C2',
               ecolor = 'C2',
               ls = '-', 
               linewidth = linewidth,
               elinewidth = elinewidth,
               marker = marker,
               markersize = markersize)


ax2.minorticks_on()
ax2.grid(which='major', linestyle='--', lw = 0.25)
ax2.grid(which='minor', linestyle=':', lw = 0.25)

ax2.legend([r'$T=780\mathregular{mV}$ Reference ',
       r'$T=790\mathregular{mV}$ Reference ',
       r'$T=800\mathregular{mV}$  Reference ',
       r'$T=790\mathregular{mV\ (DC)}-10\mathregular{mV\ (AC)}$',
       r'$T=790\mathregular{mV\ (DC)}\ +\ 0\mathregular{mV\ (AC)}$',
       r'$T=790\mathregular{mV\ (DC)}+10\mathregular{mV\ (AC)}$',
          'Static'],fontsize=6)


ax2.set_ylabel(r'$\mathregular{I_d}$ (nA)')
ax2.set_xlabel(r'$\mathregular{V_{in}}$ (V)')


ax2.set_xlim((1.5,1.54))
ax2.set_ylim((-51e-3,1))

ax2.set_xticks(np.arange(1.5,1.54,0.01))

con_g = ConnectionPatch(xyA=(1.5056,0.475), xyB=(5.5/20, 1-4), coordsA="data", coordsB="data",
                      axesA=ax2, axesB=ax1,arrowstyle = '<-',color = 'C2',lw = .5,mutation_scale=5.0)
con_o = ConnectionPatch(xyA=(1.5131,0.21), xyB=(8.5/20, 1-4), coordsA="data", coordsB="data",
                      axesA=ax2, axesB=ax1,arrowstyle = '<-',color = 'C1',lw = .5,mutation_scale=5.0)
con_b = ConnectionPatch(xyA=(1.5192,0.435), xyB=(16.5/20, 1-4), coordsA="data", coordsB="data",
                      axesA=ax2, axesB=ax1,arrowstyle = '<-',color = 'C0',lw = .5,mutation_scale=5.0)
ax2.add_artist(con_b)
ax2.add_artist(con_o)
ax2.add_artist(con_g)

ax2.xaxis.label.set_size(10)
ax2.yaxis.label.set_size(10)
ax2.xaxis.set_tick_params(labelsize=10)
ax2.yaxis.set_tick_params(labelsize=10)

# fig.tight_layout()
# fig.subplots_adjust(left =.15, right = .90)
plt.savefig('../Fig3/Fig3.png',bbox_inches = 'tight')
plt.savefig('../Fig3/Fig3.pdf',bbox_inches = 'tight')






