# -*- coding: utf-8 -*-
"""
author: F Unseld
This is the cleand up version of the script used to generate Figure S5.
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import scipy.signal as signal

#%% 

def IdNoise2VinNoise (RefData, NoiseData):
    """
    Uses the reference data to calibrate current noise (measured on the flank
    of a coulomb peak) to voltage noise on the respective gate. 
    Input:  
            RefData: DataFrame with the reference coulomb peaks. 
        Has to include the peak on which the noise was measured. Naming has
        to be Xndex: Vin; current: Id
            NoiseData: DataFrame with the noise data
        Index, current and time stamps (labeld: Vin, Id, time)
    Return:
        Dataframe with calibrated noise levels.  
    """
    # Idea: Find exact gate voltage based on matching current levels. Take 
    # average current level of the noise data and search the best match on the 
    # flank of an interpolated coulomb peak. 
    # Fit a slope through that region and calibrate current noise to 
    # gate voltage noise. 
    
    SearchRange = 4 # in samples
    
    Vsample = np.mean(NoiseData.index)

    idxmin1 = np.argmin(abs(RefData.index-Vsample))
    SamplePoint = RefData.iloc[idxmin1-SearchRange:idxmin1+SearchRange]
    
    
    newindex = np.round(np.linspace(min(SamplePoint.index), max(SamplePoint.index),SearchRange*20-9),6)
    SamplePoint_reindexed = SamplePoint.reindex(index = newindex)
    SamplePoint_reindexed.interpolate(method = 'linear', inplace = True)
    RefData_Intrplatd = RefData.append(SamplePoint_reindexed).sort_index().drop_duplicates()
    
    
    idxmin2 = np.argmin(abs(np.array(SamplePoint_reindexed.Id)-NoiseData.Id.mean()))
    calidx = RefData_Intrplatd.where(RefData_Intrplatd==SamplePoint_reindexed.iloc[idxmin2].values).dropna().index[0]
    step = abs(np.round(np.mean(np.diff(RefData.index)),5))
    np.array(RefData_Intrplatd[calidx-2.1*step:calidx+2*step].index)
    (m,c) = np.polyfit(np.array(RefData_Intrplatd[calidx-3.1*step:calidx+3*step].index), 
               np.array(RefData_Intrplatd[calidx-3.1*step:calidx+3*step].Id), 
               1)
    
    NoiseData_Gate = ((NoiseData.Id-c)/m).to_frame()
    return NoiseData_Gate

#%%
StaticRefData = DataDevA = pd.read_csv('../FigS5/StaticRef.csv',index_col=[0])
StaticNoiseData = DataDevA = pd.read_csv('../FigS5/StaticNoise.csv',index_col=[0])
FloatingRefData = DataDevA = pd.read_csv('../FigS5/FloatingRef.csv',index_col=[0])
FloatingNoiseData = DataDevA = pd.read_csv('../FigS5/FloatingNoise.csv',index_col=[0])

fsStatic = (1/StaticNoiseData.time.diff().mean())
fsFloating =(1/FloatingNoiseData.time.diff().mean())

StaticNoise_Vin = IdNoise2VinNoise(StaticRefData,StaticNoiseData)
FloatingNoise_Vin = IdNoise2VinNoise(FloatingRefData,FloatingNoiseData)

xStatic = np.array(StaticNoise_Vin.Id)
fStatic, Pxx_Static = signal.welch(xStatic, fsStatic)
PSD_Static = np.sqrt(Pxx_Static)

xFloating = np.array(FloatingNoise_Vin.Id)
fFloating, Pxx_Floating = signal.welch(xFloating, fsFloating)
PSD_Floating = np.sqrt(Pxx_Floating)

#%% Plot
fig,ax = plt.subplots(dpi = 450, figsize=(3.54,2.5))

ax.loglog(fStatic,PSD_Static,'C0', lw = 0.5,label = 'Static')
ax.loglog(fFloating,PSD_Floating,'C1', lw = 0.5,label = 'Pulsed')
ax.set_ylabel(r'$\frac{V}{\sqrt{Hz}}$')
ax.xaxis.set_major_formatter(EngFormatter(unit='Hz'))
ax.set_xlabel('Frequency')

ax.minorticks_on()
ax.grid(which='major', linestyle='--', lw = 0.25)
ax.grid(which='minor', linestyle=':', lw = 0.25)
ax.legend()
ax.set_xlim((.03,3))


fig.tight_layout(h_pad=1, w_pad=1)
plt.savefig('../FigS5/FigS5.png',bbox_inches = 'tight')
plt.savefig('../FigS5/FigS5.pdf',bbox_inches = 'tight')

































